<?php
/**
 * SkeletonCart
 * (c) 2017 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace DiscountPrice;

class Plugin extends \App\Plugin
{
    const DISCOUNT_THRESHOLD = 0;
    const DISCOUNT_VALUE = 1;
    const DISCOUNT_TYPE_PRICE = 1;
    const DISCOUNT_TYPE_RATE = 2;

    protected $fileName;
    protected $config;

    public function __construct($key)
    {
        parent::__construct($key);
        $this->fileName = __DIR__ . '/Config.php';
        $this->loadConfig();
        $this->initCallback();
    }

    public function loadConfig()
    {
        if (!is_readable($this->fileName)) {
            throw new \RuntimeException("ディスカウント設定が読み込めません。");
        }
        $this->config = include $this->fileName;

        if (!isset($this->config['type']) || !is_numeric($this->config['type'])) {
            throw new \RuntimeException("typeが設定されていません。");
        }
        if (!isset($this->config['priceTable']) || !is_array($this->config['priceTable'])) {
            throw new \RuntimeException("priceTableが設定されていません。");
        }
    }

    public function type()
    {
        return $this->config['type'];
    }

    public function priceTable()
    {
        return $this->config['priceTable'];
    }

    public function initCallback()
    {
        \App::addCallback('preUpdateSlip', array($this, 'preUpdateSlip'));
    }

    public function preUpdateSlip(\App\Slip $slip)
    {
        $discountPrice = 0;
        $discountValue = 0;
        $cart = $slip->getCart();
        $subTotal = $cart->subTotalPrice();

        foreach ($this->priceTable() as $price) {
            $discountThreshold = $price[self::DISCOUNT_THRESHOLD];
            if ($subTotal >= $discountThreshold) {
                $discountValue = $price[self::DISCOUNT_VALUE];
                if ($this->type() == self::DISCOUNT_TYPE_PRICE) {
                    $discountPrice = $discountValue;
                } elseif ($this->type() == self::DISCOUNT_TYPE_RATE) {
                    $discountPrice = intval(floor($subTotal * ($discountValue * 0.01)));
                }
            }
        }

        $subTotal -= $discountPrice;
        $slip->setSubTotal($subTotal);

        $discountProp = array(
            'type' => $this->type(),
            'price' => $discountPrice,
            'val' => $discountValue,
        );
        $slip->setExtraProperty('discount', $discountProp);

        return true;
    }
}

\App::addPlugin(new Plugin('DiscountPrice'));

